/**
 * Eventline
 * Copyright James Campbell 2017
 */
import { Middleware } from './middleware';
import { RouteBuilder } from './route-builder';
/**
 * This class is in charge of coordinating events from an event source
 * and turning them into actions to be executed based on routes.
 *
 * @export
 * @class Eventline
 */
export declare class Eventline {
    /**
     * Function for handling what happens when an exception occurs.
     *
     * @memberof Eventline
     */
    exceptionHandler: (exception: any, event: any) => void;
    /**
     * An internal array used to hold all of the
     * registered middleware for this router
     *
     * @private
     * @type {Array<Middleware>}
     * @memberof Router
     */
    private middlewares;
    /**
     * An internal array used to hold all of the
     * registered routes for this router
     *
     * @private
     * @type {Array<Route>}
     * @memberof Router
     */
    private routes;
    /**
     * Passes an event for the router to route
     *
     * @param {object} event
     * @memberof Router
     */
    route(event: object): Promise<any>;
    /**
     * Passes a component for the router to consume
     *
     * @param {(Router) => void} component
     * @memberof Router
     */
    use(component: (Router) => void): void;
    /**
     * Registers middleware with
     * the router
     *
     * @param {Middleware} middleware
     * @memberof Router
     */
    registerMiddleware(middleware: Middleware): void;
    /**
     * Registers routes for the router
     *
     * @param {*} patterns
     * @returns RouteBuilder
     * @memberof Router
     */
    on(...patterns: any[]): RouteBuilder;
    /**
     * An internal method for running middleware
     * flow for the runtime
     *
     * @param {string} type
     * @param {any} event
     * @returns {Promise}
     * @memberof Router
     */
    private runMiddleware(type, event);
    /**
     * An internal method for taking an event
     * finding a matching router and triggering it.
     *
     * @param {*} event
     * @return Promise
     * @memberof Router
     */
    private handleEvent(event);
    /**
     * An internal method which finds
     * a matching route given an event
     *
     * Each route is evaluated in the
     * order it was declared.
     *
     * @param {*} event
     * @returns
     * @memberof Router
     */
    private routeForEvent(event);
}
